import os
from datasets import load_dataset
from transformers import AutoTokenizer
from transformers import LlamaTokenizerFast

def calc_tokens(dataset_path:str, token_path:str,epoch:int):
    try:
        tokenizer = AutoTokenizer.from_pretrained(token_path, trust_remote_code=True)
    except Exception as e:
        tokenizer = LlamaTokenizerFast.from_pretrained(token_path)
    file_list = [
        os.path.join(dataset_path, filename)
        for filename in os.listdir(dataset_path)
        if filename.endswith((".json", ".jsonl"))
    ]
    hf_dataset = load_dataset(path='json', data_files=file_list)

    def count_tokens(example):
        len_token_cnts = []
        for instruction,input,output in zip(example['instruction'],example['input'],example['output']):
            full_prompt = instruction + input + output
            token_cnt = tokenizer.tokenize(full_prompt)
            len_token_cnt = len(token_cnt)
            len_token_cnts.append(len_token_cnt)
        return {'total_tokens':len_token_cnts}
    
    token_dict = hf_dataset.map(count_tokens,
                                remove_columns=['instruction', 'input', 'output'],
                                batched=True)

    col_token = token_dict['train']['total_tokens']
    return sum(col_token)